import uuid

from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, DeclarativeBase


class Base(DeclarativeBase):
    id: Mapped[uuid.UUID] = mapped_column(
        UUID(as_uuid=True),
        primary_key=True,
        default=uuid.uuid4
    )




from typing import List, TYPE_CHECKING

from sqlalchemy.orm import Mapped, relationship

from ._base import Base

if TYPE_CHECKING:
    from .user import User
    from .meet import Meet


class Club(Base):
    __tablename__ = "clubs"

    users: Mapped[List["User"]] = relationship(
        "User", back_populates="club", passive_deletes=True
    )

    meets: Mapped[List["Meet"]] = relationship(
        "Meet", back_populates="club"
    )



from typing import TYPE_CHECKING

from sqlalchemy import String, ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ._base import Base

if TYPE_CHECKING:
    from .club import Club
    from .meet import Meet
    from .meet_attendee import MeetAttendee


class User(Base):
    __tablename__ = "users"

    # Clerk external auth ID
    clerk_id: Mapped[str] = mapped_column(
        String,
        unique=True,
        nullable=False,
        index=True
    )

    club_id: Mapped[UUID] = mapped_column(
        UUID(as_uuid=True),
        ForeignKey("clubs.id", ondelete="SET NULL"),
        nullable=True,
        index=True,
    )

    club: Mapped["Club"] = relationship(
        "Club", back_populates="users"
    )

    meets: Mapped[list["Meet"]] = relationship(
        "Meet", back_populates="user"
    )

    meet_attendances: Mapped[list["MeetAttendee"]] = relationship(
        "MeetAttendee", back_populates="user"
    )



from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ._base import Base

if TYPE_CHECKING:
    from .meet import Meet
    from .user import User


class MeetAttendee(Base):
    __tablename__ = "meet_attendees"

    meet_id: Mapped[UUID] = mapped_column(
        UUID(as_uuid=True),
        ForeignKey("meets.id"),
        nullable=False,
        index=True,
    )

    user_id: Mapped[UUID] = mapped_column(
        UUID(as_uuid=True),
        ForeignKey("users.id"),
        nullable=False,
        index=True,
    )

    meet: Mapped["Meet"] = relationship(
        "Meet", back_populates="attendees"
    )

    user: Mapped["User"] = relationship(
        "User", back_populates="meet_attendances"
    )



from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ._base import Base

if TYPE_CHECKING:
    from .club import Club
    from .meet_attendee import MeetAttendee
    from .user import User


class Meet(Base):
    __tablename__ = "meets"

    user_id: Mapped[UUID] = mapped_column(
        UUID(as_uuid=True),
        ForeignKey("users.id"),
        nullable=False,
        index=True,
    )

    club_id: Mapped[UUID] = mapped_column(
        UUID(as_uuid=True),
        ForeignKey("clubs.id"),
        nullable=False,
        index=True,
    )

    user: Mapped["User"] = relationship(
        "User", back_populates="meets"
    )

    club: Mapped["Club"] = relationship(
        "Club", back_populates="meets"
    )

    attendees: Mapped[list["MeetAttendee"]] = relationship(
        "MeetAttendee", back_populates="meet"
    )
